@@ -1,29 +1,32 @@
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "MyErrorsHandlers.h"
 
 #include <stdio.h>
 #include <signal.h>
+
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
-#if defined(DARWIN)
-#include <sys/wait.h>
-#else
-#include <wait.h>
 #endif
 
-#include "wbx.h"
-#include "erbi.h"
-#include "py.h"
+#ifdef HAVE_WAIT_H
+#include <wait.h>
+#else
+#include <sys/wait.h>
+#endif
 
-#if defined(DARWIN)
+#include "ime.h"
+#include "tools.h"
+
+#ifndef SIGUNUSED
 #define SIGUNUSED 32
 #endif
 
 XErrorHandler   oldXErrorHandler;
 
-extern BYTE     iWBChanged;
-extern BYTE		iEBChanged;
-extern BYTE     iNewPYPhraseCount;
-extern BYTE     iOrderCount;
-extern BYTE     iNewFreqCount;
+extern Bool     bLumaQQ;
 
 void SetMyExceptionHandler (void)
 {
@@ -35,19 +38,20 @@ void SetMyExceptionHandler (void)
 
 void OnException (int signo)
 {
-    //   int             status;
     fprintf (stderr, "\nFCITX -- Get Signal No.: %d\n", signo);
-	
-    if (iWBChanged)
-	SaveWubiDict ();
-	if (iEBChanged)
-	SaveErbiDict ();
-    if (iNewPYPhraseCount)
-	SavePYUserPhrase ();
-    if (iOrderCount)
-	SavePYIndex ();
-    if (iNewFreqCount)
-	SavePYFreq ();
+
+    if (signo == SIGHUP) {
+	SetIM ();
+	LoadConfig (False);
+
+	if (bLumaQQ)
+	    ConnectIDResetReset ();
+
+	return;
+    }
+
+    if (signo != SIGSEGV)	//SIGSEGVԼ⣬ʱִбܻ뷨ļ
+	SaveIM ();
 
     if (signo != SIGCHLD && signo != SIGQUIT && signo != SIGWINCH) {
 	fprintf (stderr, "FCITX -- Exit Signal No.: %d\n\n", signo);
@@ -62,20 +66,11 @@ void SetMyXErrorHandler (void)
 
 int MyXErrorHandler (Display * dpy, XErrorEvent * event)
 {
-    char            str[1025];
+    char            str[256];
 
-    if (iWBChanged)
-	SaveWubiDict ();
-	if (iEBChanged)
-	SaveErbiDict ();
-    if (iNewPYPhraseCount)
-	SavePYUserPhrase ();
-    if (iOrderCount)
-	SavePYIndex ();
-    if (iNewFreqCount)
-	SavePYFreq ();
+    SaveIM ();
 
-    XGetErrorText (dpy, event->error_code, str, 1024);
+    XGetErrorText (dpy, event->error_code, str, 255);
     fprintf (stderr, "fcitx: %s\n", str);
 
     if (event->error_code != 3 && event->error_code != BadMatch)	// xterm will generate 3
