@@ -1,3 +1,7 @@
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <sys/stat.h>
 #include <limits.h>
@@ -5,15 +9,17 @@
 #include <string.h>
 #include <unistd.h>
 
+#ifdef HAVE_MACHINE_ENDIAN_H
+#include <machine/endian.h>
+#endif
+
 #include "ime.h"
 #include "tools.h"
 #include "InputWindow.h"
 #include "py.h"
 #include "PYFA.h"
 #include "pyParser.h"
-#include "wbx.h"
 #include "sp.h"
-#include "SetLocale.h"
 
 int             iPYFACount;
 PYFA           *PYFAList;
@@ -21,7 +27,7 @@ uint            iCounter = 0;
 Bool            bPYBaseDictLoaded = False;
 Bool            bPYOtherDictLoaded = False;
 
-Bool            bSingleHZMode = True;	//£ֻе룬дƥ䡢ɼ棬Zƴ
+Bool            bSingleHZMode = True;	//£ֻе룬дƥ䡢ɼ棬뷨еƴ
 
 PyFreq         *pyFreq = NULL, *pCurFreq = NULL;
 uint            iPYFreqCount = 0;
@@ -30,8 +36,8 @@ char            strFindString[MAX_USER_INPUT + 1];
 ParsePYStruct   findMap;
 int             iPYInsertPoint = 0;
 
-char            strPYLegendSource[MAX_WORDS_USER_INPUT * 2 + 1] = "";
-char            strPYLegendMap[MAX_WORDS_USER_INPUT * 2 + 1] = "";
+char            strPYLegendSource[MAX_WORDS_USER_INPUT * 2 + 1] = "\0";
+char            strPYLegendMap[MAX_WORDS_USER_INPUT * 2 + 1] = "\0";
 PyBase         *pyBaseForLengend;
 PYLegendCandWord PYLegendCandWords[MAX_CAND_WORD];
 
@@ -48,9 +54,11 @@ ADJUSTORDER     baseOrder = AD_FREQ;
 ADJUSTORDER     phraseOrder = AD_FAST;
 ADJUSTORDER     freqOrder = AD_NO;
 
-BYTE            iNewPYPhraseCount = 0;
-BYTE            iOrderCount = 0;
-BYTE            iNewFreqCount = 0;
+INT8            iNewPYPhraseCount = 0;
+INT8            iOrderCount = 0;
+INT8            iNewFreqCount = 0;
+
+INT8            iYCDZ = 0;
 
 HOTKEYS         hkPYAddFreq[HOT_KEY_COUNT] = { CTRL_8, 0 };
 Bool            bIsPYAddFreq = False;
@@ -88,30 +96,25 @@ extern int      iLegendCandPageCount;
 extern int      iCurrentLegendCandPage;
 extern Bool     bDisablePagingInLegend;
 
-#if defined(DARWIN)
-/* function to reverse byte order for integer
-this is required for Mac machine*/
-int ReverseInt (unsigned int pc_int)
+void PYInit (void)
 {
-    int mac_int;
-    unsigned char * p;
-
-    mac_int = pc_int;
-    p = (unsigned char*) &pc_int;
-    mac_int = (p[3] << 24) + (p[2] << 16) + (p[1] << 8) + p[0];
-    return mac_int;
+    bSP = False;
 }
-#endif
 
 Bool LoadPYBaseDict (void)
 {
     FILE           *fp;
     char            strPath[PATH_MAX];
-    #if defined(DARWIN)
+
+/*
+#if defined(DARWIN)
     unsigned int    i, j, iLen;
-    #else
+#else*/
     int             i, j, iLen;
-    #endif
+
+/*
+#endif
+*/
 
     strcpy (strPath, PKGDATADIR "/data/");
     strcat (strPath, PY_BASE_FILE);
@@ -121,9 +124,6 @@ Bool LoadPYBaseDict (void)
 
     fread (&iPYFACount, sizeof (int), 1, fp);
     PYFAList = (PYFA *) malloc (sizeof (PYFA) * iPYFACount);
-    if (!PYFAList)
-	return False;
-
     for (i = 0; i < iPYFACount; i++) {
 	fread (PYFAList[i].strMap, sizeof (char) * 2, 1, fp);
 	PYFAList[i].strMap[2] = '\0';
@@ -172,23 +172,26 @@ Bool LoadPYOtherDict (void)
     strcat (strPath, PY_PHRASE_FILE);
     fp = fopen (strPath, "rb");
     if (!fp)
-	fprintf (stderr, "\nûϵͳļ\n");
+	fprintf (stderr, "\nûϵͳʿļ\n");
     else {
 	while (!feof (fp)) {
 	    if (!fread (&i, sizeof (int), 1, fp))
-		#if defined(DARWIN)
-		i = ReverseInt(i);
-		#endif
 		break;
+/*
+#if defined(DARWIN)
+            i = ReverseInt (i);
+#endif
+*/
 	    if (!fread (strBase, sizeof (char) * 2, 1, fp))
 		break;
 	    strBase[2] = '\0';
 	    if (!fread (&k, sizeof (int), 1, fp))
-		#if defined(DARWIN)
-		k = ReverseInt(k);
-		#endif
 		break;
-
+/*
+#if defined(DARWIN)
+            k = ReverseInt (k);
+#endif
+*/
 	    j = GetBaseIndex (i, strBase);
 
 	    PYFAList[i].pyBase[j].iPhrase = k;
@@ -213,10 +216,6 @@ Bool LoadPYOtherDict (void)
     //濪ʼȡûʿ
     strcpy (strPath, (char *) getenv ("HOME"));
     strcat (strPath, "/.fcitx/");
-
-    if (access (strPath, 0))
-	mkdir (strPath, S_IRWXU);
-
     strcat (strPath, PY_USERPHRASE_FILE);
     fp = fopen (strPath, "rb");
     if (fp) {
@@ -700,7 +699,62 @@ INPUT_RETURN_VALUE DoPYInput (int iKey)
 	else {
 	    if (bIsPYAddFreq || bIsPYDelFreq || bIsPYDelUserPhr)
 		return IRV_DO_NOTHING;
-	    return IRV_TO_PROCESS;
+
+	    //ʵԴʶ
+	    if (iCandWordCount) {
+		if (iKey == '[' || iKey == ']') {
+		    if (PYCandWords[iYCDZ].iWhich == PY_CAND_USERPHRASE || PYCandWords[iYCDZ].iWhich == PY_CAND_SYMPHRASE) {
+			char           *pBase, *pPhrase;
+
+			pBase = PYFAList[PYCandWords[iYCDZ].cand.phrase.iPYFA].pyBase[PYCandWords[iYCDZ].cand.phrase.iBase].strHZ;
+			pPhrase = PYCandWords[iYCDZ].cand.phrase.phrase->strPhrase;
+
+			if (iKey == '[')
+			    strcpy (strStringGet, pBase);
+			else {
+			    strncpy (strStringGet, pPhrase, 2);
+			    strStringGet[2] = '\0';
+			}
+			uMessageDown = 0;
+			return IRV_GET_CANDWORDS;
+		    }
+		}
+		else if ( !bIsInLegend ) {
+		    val = -1;
+		    switch (iKey) {
+		    case ')':
+			val++;
+		    case '(':
+			val++;
+		    case '*':
+			val++;
+		    case '&':
+			val++;
+		    case '^':
+			val++;
+		    case '%':
+			val++;
+		    case '$':
+			val++;
+		    case '#':
+			val++;
+		    case '@':
+			val++;
+		    case '!':
+			val++;
+		    }
+
+		    if (val != -1 && val < iCandWordCount) {
+			iYCDZ = val;
+			PYCreateCandString ();
+			return IRV_DISPLAY_CANDWORDS;
+		    }
+
+		    return IRV_TO_PROCESS;
+		}
+	    }
+	    else
+		return IRV_TO_PROCESS;
 	}
     }
 
@@ -733,7 +787,7 @@ INPUT_RETURN_VALUE DoPYInput (int iKey)
 	return PYGetCandWords (SM_FIRST);
     }
 
-    return val;
+    return (INPUT_RETURN_VALUE) val;
 }
 
 /*
@@ -826,11 +880,7 @@ void UpdateFindString (void)
 
 INPUT_RETURN_VALUE PYGetCandWords (SEARCH_MODE mode)
 {
-    char            str[2];
     int             iVal;
-    char           *pBase = NULL, *pPhrase;
-    int             iType;
-
 
     if (findMap.iMode == PARSE_ERROR || (bSingleHZMode && findMap.iHZCount > 1)) {
 	uMessageDown = 0;
@@ -847,6 +897,7 @@ INPUT_RETURN_VALUE PYGetCandWords (SEARCH_MODE mode)
 	iCurrentCandPage = 0;
 	iCandPageCount = 0;
 	iCandWordCount = 0;
+	iYCDZ = 0;
 
 	PYResetFlags ();
 
@@ -858,9 +909,8 @@ INPUT_RETURN_VALUE PYGetCandWords (SEARCH_MODE mode)
 	    pCurFreq = pCurFreq->next;
 	}
 
-	if (!bSingleHZMode && bPYCreateAuto) {
+	if (!bSingleHZMode && bPYCreateAuto)
 	    PYCreateAuto ();
-	}
     }
     else {
 	if (!iCandPageCount)
@@ -885,7 +935,7 @@ INPUT_RETURN_VALUE PYGetCandWords (SEARCH_MODE mode)
     }
 
     if (!(pCurFreq && pCurFreq->bIsSym)) {
-	if (!iCurrentCandPage && strPYAuto[0]) {
+	if (!iCurrentCandPage && strPYAuto[0] && !bSingleHZMode) {
 	    iCandWordCount = 1;
 	    PYCandWords[0].iWhich = PY_CAND_AUTO;
 	}
@@ -902,7 +952,19 @@ INPUT_RETURN_VALUE PYGetCandWords (SEARCH_MODE mode)
     else
 	PYGetCandWordsBackward ();
 
-    str[1] = '\0';
+    PYCreateCandString ();
+
+    return IRV_DISPLAY_CANDWORDS;
+}
+
+void PYCreateCandString (void)
+{
+    char            str[3];
+    char           *pBase = NULL, *pPhrase;
+    int             iType, iVal;
+
+    str[1] = '.';
+    str[2] = '\0';
     uMessageDown = 0;
 
     for (iVal = 0; iVal < iCandWordCount; iVal++) {
@@ -911,9 +973,6 @@ INPUT_RETURN_VALUE PYGetCandWords (SEARCH_MODE mode)
 	else
 	    str[0] = iVal + 1 + '0';
 	strcpy (messageDown[uMessageDown].strMsg, str);
-#ifdef _USE_XFT
-	strcat (messageDown[uMessageDown].strMsg, " ");
-#endif
 	messageDown[uMessageDown++].type = MSG_INDEX;
 
 	iType = MSG_OTHER;
@@ -953,13 +1012,11 @@ INPUT_RETURN_VALUE PYGetCandWords (SEARCH_MODE mode)
 	    strcat (messageDown[uMessageDown].strMsg, " ");
 #endif
 	}
-	if (PYCandWords[iVal].iWhich != PY_CAND_AUTO && iVal == 0)
+	if (PYCandWords[iVal].iWhich != PY_CAND_AUTO && iVal == iYCDZ)
 	    iType = MSG_FIRSTCAND;
 
-	messageDown[uMessageDown++].type = iType;
+	messageDown[uMessageDown++].type = (MSG_TYPE) iType;
     }
-
-    return IRV_DISPLAY_CANDWORDS;
 }
 
 void PYSetCandWordsFlag (Bool flag)
@@ -1038,7 +1095,7 @@ void PYCreateAuto (void)
 			    if (!val && iMatchedLength == (findMap.iHZCount - 1) * 2)
 				return;
 			    if (!val || (val && (strlen (phrase->strMap) == iMatchedLength))) {
-				if (CheckLocale (phrase->strPhrase) && CheckLocale (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ)) {
+				if (CheckHZCharset (phrase->strPhrase) && CheckHZCharset (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ)) {
 				    if (!phraseSelected) {
 					baseSelected = &(PYFAList[candPos.iPYFA].pyBase[candPos.iBase]);
 					pPYFA = &PYFAList[candPos.iPYFA];
@@ -1070,7 +1127,7 @@ void PYCreateAuto (void)
 		if (!Cmp2Map (PYFAList[candPos.iPYFA].strMap, str)) {
 		    for (candPos.iBase = 0; candPos.iBase < PYFAList[candPos.iPYFA].iBase; candPos.iBase++) {
 			for (candPos.iPhrase = 0; candPos.iPhrase < PYFAList[candPos.iPYFA].pyBase[candPos.iBase].iPhrase; candPos.iPhrase++) {
-			    if (CheckLocale (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].strPhrase) && CheckLocale (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ)) {
+			    if (CheckHZCharset (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].strPhrase) && CheckHZCharset (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ)) {
 				val = CmpMap (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].strMap, strMap, &iMatchedLength);
 				if (!val && iMatchedLength == (findMap.iHZCount - 1) * 2)
 				    return;
@@ -1114,8 +1171,9 @@ void PYCreateAuto (void)
 	    for (candPos.iPYFA = 0; candPos.iPYFA < iPYFACount; candPos.iPYFA++) {
 		if (!Cmp2Map (PYFAList[candPos.iPYFA].strMap, str)) {
 		    for (candPos.iBase = 0; candPos.iBase < PYFAList[candPos.iPYFA].iBase; candPos.iBase++) {
-			if (CheckLocale (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ))
-			    if ((int) (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].iHit) > val) {
+			if (CheckHZCharset (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ))
+			    if ((int)
+				(PYFAList[candPos.iPYFA].pyBase[candPos.iBase].iHit) > val) {
 				val = PYFAList[candPos.iPYFA].pyBase[candPos.iBase].iHit;
 				baseSelected = &(PYFAList[candPos.iPYFA].pyBase[candPos.iBase]);
 				pPYFA = &PYFAList[candPos.iPYFA];
@@ -1146,7 +1204,6 @@ char           *PYGetCandWord (int iIndex)
     char            strHZString[MAX_WORDS_USER_INPUT * 2 + 1];
     int             iLen;
 
-
     if (!iCandWordCount)
 	return NULL;
 
@@ -1163,7 +1220,7 @@ char           *PYGetCandWord (int iIndex)
 	pBase = PYFAList[PYCandWords[iIndex].cand.base.iPYFA].pyBase[PYCandWords[iIndex].cand.base.iBase].strHZ;
 	pBaseMap = PYFAList[PYCandWords[iIndex].cand.base.iPYFA].strMap;
 	pIndex = &(PYFAList[PYCandWords[iIndex].cand.base.iPYFA].pyBase[PYCandWords[iIndex].cand.base.iBase].iIndex);
-	PYFAList[PYCandWords[iIndex].cand.base.iPYFA].pyBase[PYCandWords[iIndex].cand.base.iBase].iHit = iCounter++;
+	PYFAList[PYCandWords[iIndex].cand.base.iPYFA].pyBase[PYCandWords[iIndex].cand.base.iBase].iHit++;
 	iOrderCount++;
 	break;
     case PY_CAND_SYMPHRASE:	//ϵͳ    
@@ -1173,7 +1230,7 @@ char           *PYGetCandWord (int iIndex)
 	pPhrase = PYCandWords[iIndex].cand.phrase.phrase->strPhrase;
 	pPhraseMap = PYCandWords[iIndex].cand.phrase.phrase->strMap;
 	pIndex = &(PYCandWords[iIndex].cand.phrase.phrase->iIndex);
-	PYCandWords[iIndex].cand.phrase.phrase->iHit = iCounter++;
+	PYCandWords[iIndex].cand.phrase.phrase->iHit++;
 	iOrderCount++;
 	break;
     case PY_CAND_FREQ:		//ǳ
@@ -1331,7 +1388,7 @@ Bool PYCheckNextCandPage (void)
 			for (candPos.iPhrase = 0; candPos.iPhrase < PYFAList[candPos.iPYFA].pyBase[candPos.iBase].iUserPhrase; candPos.iPhrase++) {
 			    val = CmpMap (phrase->strMap, strMap, &iMatchedLength);
 			    if (!val || (val && (strlen (phrase->strMap) == iMatchedLength))) {
-				if (CheckLocale (phrase->strPhrase) && CheckLocale (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ)) {
+				if (CheckHZCharset (phrase->strPhrase) && CheckHZCharset (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ)) {
 				    if (!phrase->flag)
 					return True;
 				}
@@ -1349,7 +1406,7 @@ Bool PYCheckNextCandPage (void)
 			    if (!PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].flag) {
 				val = CmpMap (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].strMap, strMap, &iMatchedLength);
 				if (!val || (val && (strlen (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].strMap) == iMatchedLength))) {
-				    if (CheckLocale (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].strPhrase) && CheckLocale (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ))
+				    if (CheckHZCharset (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].strPhrase) && CheckHZCharset (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ))
 					return True;
 				}
 			    }
@@ -1372,7 +1429,7 @@ Bool PYCheckNextCandPage (void)
 	    if (!Cmp2Map (PYFAList[candPos.iPYFA].strMap, str)) {
 		for (candPos.iBase = 0; candPos.iBase < PYFAList[candPos.iPYFA].iBase; candPos.iBase++) {
 		    if (!PYFAList[candPos.iPYFA].pyBase[candPos.iBase].flag) {
-			if (CheckLocale (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ) && !PYIsInFreq (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ))
+			if (CheckHZCharset (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ) && !PYIsInFreq (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ))
 			    return True;
 		    }
 		}
@@ -1405,10 +1462,10 @@ void PYGetPhraseCandWords (SEARCH_MODE mode)
     for (candPos.iPYFA = 0; candPos.iPYFA < iPYFACount; candPos.iPYFA++) {
 	if (!Cmp2Map (PYFAList[candPos.iPYFA].strMap, str)) {
 	    for (candPos.iBase = 0; candPos.iBase < PYFAList[candPos.iPYFA].iBase; candPos.iBase++) {
-		if (CheckLocale (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ)) {
+		if (CheckHZCharset (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ)) {
 		    phrase = PYFAList[candPos.iPYFA].pyBase[candPos.iBase].userPhrase->next;
 		    for (candPos.iPhrase = 0; candPos.iPhrase < PYFAList[candPos.iPYFA].pyBase[candPos.iBase].iUserPhrase; candPos.iPhrase++) {
-			if (CheckLocale (phrase->strPhrase)) {
+			if (CheckHZCharset (phrase->strPhrase)) {
 			    val = CmpMap (phrase->strMap, strMap, &iMatchedLength);
 			    if (!val || (val && (strlen (phrase->strMap) == iMatchedLength))) {
 				if ((mode != SM_PREV && !phrase->flag)
@@ -1430,7 +1487,7 @@ void PYGetPhraseCandWords (SEARCH_MODE mode)
 	if (!Cmp2Map (PYFAList[candPos.iPYFA].strMap, str)) {
 	    for (candPos.iBase = 0; candPos.iBase < PYFAList[candPos.iPYFA].iBase; candPos.iBase++) {
 		for (candPos.iPhrase = 0; candPos.iPhrase < PYFAList[candPos.iPYFA].pyBase[candPos.iBase].iPhrase; candPos.iPhrase++) {
-		    if (CheckLocale (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].strPhrase) && CheckLocale (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ)) {
+		    if (CheckHZCharset (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].strPhrase) && CheckHZCharset (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ)) {
 			val = CmpMap (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].strMap, strMap, &iMatchedLength);
 			if (!val || (val && (strlen (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].strMap) == iMatchedLength))) {
 			    if ((mode != SM_PREV && !PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].flag)
@@ -1518,7 +1575,7 @@ Bool PYAddPhraseCandWord (PYCandIndex pos, PyPhrase * phrase, SEARCH_MODE mode,
 			break;
 		    }
 		    else if (strlen (PYCandWords[i].cand.phrase.phrase->strPhrase) == strlen (phrase->strPhrase)) {
-			if (phrase->iHit <= PYCandWords[i].cand.phrase.phrase->iHit) {
+			if (phrase->iIndex <= PYCandWords[i].cand.phrase.phrase->iIndex) {
 			    i++;
 			    break;
 			}
@@ -1541,7 +1598,7 @@ Bool PYAddPhraseCandWord (PYCandIndex pos, PyPhrase * phrase, SEARCH_MODE mode,
 		    if (strlen (phrase->strPhrase) > strlen (PYCandWords[i].cand.phrase.phrase->strPhrase))
 			break;
 		    else if (strlen (PYCandWords[i].cand.phrase.phrase->strPhrase) == strlen (phrase->strPhrase)) {
-			if (phrase->iHit > PYCandWords[i].cand.phrase.phrase->iHit)
+			if (phrase->iIndex > PYCandWords[i].cand.phrase.phrase->iIndex)
 			    break;
 		    }
 		}
@@ -1565,7 +1622,7 @@ Bool PYAddPhraseCandWord (PYCandIndex pos, PyPhrase * phrase, SEARCH_MODE mode,
 			break;
 		    }
 		    else if (strlen (PYCandWords[i].cand.phrase.phrase->strPhrase) == strlen (phrase->strPhrase)) {
-			if (PYCandWords[i].cand.phrase.phrase->iIndex >= phrase->iIndex)
+			if (PYCandWords[i].cand.phrase.phrase->iHit >= phrase->iHit)
 			    i++;
 			break;
 		    }
@@ -1585,7 +1642,7 @@ Bool PYAddPhraseCandWord (PYCandIndex pos, PyPhrase * phrase, SEARCH_MODE mode,
 		if (PYCandWords[i].iWhich == PY_CAND_USERPHRASE || PYCandWords[i].iWhich == PY_CAND_SYMPHRASE) {
 		    if (strlen (PYCandWords[i].cand.phrase.phrase->strPhrase) < strlen (phrase->strPhrase))
 			break;
-		    else if ((strlen (PYCandWords[i].cand.phrase.phrase->strPhrase) == strlen (phrase->strPhrase)) && (PYCandWords[i].cand.phrase.phrase->iIndex < phrase->iIndex))
+		    else if ((strlen (PYCandWords[i].cand.phrase.phrase->strPhrase) == strlen (phrase->strPhrase)) && (PYCandWords[i].cand.phrase.phrase->iHit < phrase->iHit))
 			break;
 		}
 	    }
@@ -1746,7 +1803,7 @@ void PYGetBaseCandWords (SEARCH_MODE mode)
     for (candPos.iPYFA = 0; candPos.iPYFA < iPYFACount; candPos.iPYFA++) {
 	if (!Cmp2Map (PYFAList[candPos.iPYFA].strMap, str)) {
 	    for (candPos.iBase = 0; candPos.iBase < PYFAList[candPos.iPYFA].iBase; candPos.iBase++) {
-		if (CheckLocale (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ)) {
+		if (CheckHZCharset (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ)) {
 		    if ((mode != SM_PREV && !PYFAList[candPos.iPYFA].pyBase[candPos.iBase].flag) || (mode == SM_PREV && PYFAList[candPos.iPYFA].pyBase[candPos.iBase].flag)) {
 			if (!PYIsInFreq (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].strHZ)) {
 			    if (!PYAddBaseCandWord (candPos, mode))
@@ -1770,10 +1827,6 @@ Bool PYAddBaseCandWord (PYCandIndex pos, SEARCH_MODE mode)
     int             i = 0, j;
     int             iStart = 0;
 
-/*    char           *s;
-
-    s = PYFAList[pos.iPYFA].pyBase[pos.iBase].strHZ;
-  */
     switch (baseOrder) {
     case AD_NO:
 	if (mode == SM_PREV) {
@@ -1789,39 +1842,6 @@ Bool PYAddBaseCandWord (PYCandIndex pos, SEARCH_MODE mode)
 	}
 	break;
     case AD_FAST:
-	if (mode == SM_PREV) {
-	    for (i = (iCandWordCount - 1); i >= 0; i--) {
-		if (PYCandWords[i].iWhich == PY_CAND_AUTO || PYCandWords[i].iWhich == PY_CAND_FREQ) {
-		    iStart = i + 1;
-		    i++;
-		    break;
-		}
-		else if (PYCandWords[i].iWhich == PY_CAND_BASE && (PYFAList[PYCandWords[i].cand.base.iPYFA].pyBase[PYCandWords[i].cand.base.iBase].iHit >= PYFAList[pos.iPYFA].pyBase[pos.iBase].iHit)) {
-		    i++;
-		    break;
-		}
-	    }
-
-	    if (i < 0) {
-		if (iCandWordCount == iMaxCandWord)
-		    return True;
-		i = 0;
-	    }
-	    else if (iCandWordCount == iMaxCandWord)
-		i--;
-	}
-	else {
-	    for (i = 0; i < iCandWordCount; i++) {
-		if (PYCandWords[i].iWhich == PY_CAND_BASE && (PYFAList[PYCandWords[i].cand.base.iPYFA].pyBase[PYCandWords[i].cand.base.iBase].iHit < PYFAList[pos.iPYFA].pyBase[pos.iBase].iHit))
-		    break;
-	    }
-
-	    if (i == iMaxCandWord)
-		return True;
-	}
-
-	break;
-    case AD_FREQ:
 	if (mode == SM_PREV) {
 	    for (i = (iCandWordCount - 1); i >= 0; i--) {
 		if (PYCandWords[i].iWhich == PY_CAND_AUTO || PYCandWords[i].iWhich == PY_CAND_FREQ) {
@@ -1848,6 +1868,39 @@ Bool PYAddBaseCandWord (PYCandIndex pos, SEARCH_MODE mode)
 		if (PYCandWords[i].iWhich == PY_CAND_BASE && (PYFAList[PYCandWords[i].cand.base.iPYFA].pyBase[PYCandWords[i].cand.base.iBase].iIndex < PYFAList[pos.iPYFA].pyBase[pos.iBase].iIndex))
 		    break;
 	    }
+
+	    if (i == iMaxCandWord)
+		return True;
+	}
+
+	break;
+    case AD_FREQ:
+	if (mode == SM_PREV) {
+	    for (i = (iCandWordCount - 1); i >= 0; i--) {
+		if (PYCandWords[i].iWhich == PY_CAND_AUTO || PYCandWords[i].iWhich == PY_CAND_FREQ) {
+		    iStart = i + 1;
+		    i++;
+		    break;
+		}
+		else if (PYCandWords[i].iWhich == PY_CAND_BASE && (PYFAList[PYCandWords[i].cand.base.iPYFA].pyBase[PYCandWords[i].cand.base.iBase].iHit >= PYFAList[pos.iPYFA].pyBase[pos.iBase].iHit)) {
+		    i++;
+		    break;
+		}
+	    }
+
+	    if (i < 0) {
+		if (iCandWordCount == iMaxCandWord)
+		    return True;
+		i = 0;
+	    }
+	    else if (iCandWordCount == iMaxCandWord)
+		i--;
+	}
+	else {
+	    for (i = 0; i < iCandWordCount; i++) {
+		if (PYCandWords[i].iWhich == PY_CAND_BASE && (PYFAList[PYCandWords[i].cand.base.iPYFA].pyBase[PYCandWords[i].cand.base.iBase].iHit < PYFAList[pos.iPYFA].pyBase[pos.iBase].iHit))
+		    break;
+	    }
 	    if (i == iMaxCandWord)
 		return True;
 	}
@@ -1973,37 +2026,6 @@ Bool PYAddFreqCandWord (HZ * hz, char *strPY, SEARCH_MODE mode)
 	}
 	break;
     case AD_FAST:
-	if (mode == SM_PREV) {
-	    for (i = (iCandWordCount - 1); i >= 0; i--) {
-		if (PYCandWords[i].iWhich == PY_CAND_AUTO) {
-		    iStart = i + 1;
-		    i++;
-		    break;
-		}
-		else if (PYCandWords[i].iWhich == PY_CAND_FREQ && (hz->iHit <= PYCandWords[i].cand.freq.hz->iHit)) {
-		    i++;
-		    break;
-		}
-	    }
-
-	    if (i < 0) {
-		if (iCandWordCount == iMaxCandWord)
-		    return True;
-		i = 0;
-	    }
-	    else if (iCandWordCount == iMaxCandWord)
-		i--;
-	}
-	else {
-	    for (i = 0; i < iCandWordCount; i++) {
-		if (PYCandWords[i].iWhich == PY_CAND_FREQ && (hz->iHit > PYCandWords[i].cand.freq.hz->iHit))
-		    break;
-	    }
-	    if (i == iMaxCandWord)
-		return True;
-	}
-	break;
-    case AD_FREQ:
 	if (mode == SM_PREV) {
 	    for (i = (iCandWordCount - 1); i >= 0; i--) {
 		if (PYCandWords[i].iWhich == PY_CAND_AUTO) {
@@ -2034,6 +2056,37 @@ Bool PYAddFreqCandWord (HZ * hz, char *strPY, SEARCH_MODE mode)
 		return True;
 	}
 	break;
+    case AD_FREQ:
+	if (mode == SM_PREV) {
+	    for (i = (iCandWordCount - 1); i >= 0; i--) {
+		if (PYCandWords[i].iWhich == PY_CAND_AUTO) {
+		    iStart = i + 1;
+		    i++;
+		    break;
+		}
+		else if (PYCandWords[i].iWhich == PY_CAND_FREQ && (hz->iHit <= PYCandWords[i].cand.freq.hz->iHit)) {
+		    i++;
+		    break;
+		}
+	    }
+
+	    if (i < 0) {
+		if (iCandWordCount == iMaxCandWord)
+		    return True;
+		i = 0;
+	    }
+	    else if (iCandWordCount == iMaxCandWord)
+		i--;
+	}
+	else {
+	    for (i = 0; i < iCandWordCount; i++) {
+		if (PYCandWords[i].iWhich == PY_CAND_FREQ && (hz->iHit > PYCandWords[i].cand.freq.hz->iHit))
+		    break;
+	    }
+	    if (i == iMaxCandWord)
+		return True;
+	}
+	break;
     }
 
     if (mode == SM_PREV) {
@@ -2228,17 +2281,18 @@ void SavePYUserPhrase (void)
     int             i, j, k;
     int             iTemp;
     char            strPath[PATH_MAX];
+    char            strPathTemp[PATH_MAX];
     FILE           *fp;
     PyPhrase       *phrase;
 
-    strcpy (strPath, (char *) getenv ("HOME"));
-    strcat (strPath, "/.fcitx/");
-    if (access (strPath, 0))
-	mkdir (strPath, S_IRWXU);
-    strcat (strPath, PY_USERPHRASE_FILE);
-    fp = fopen (strPath, "wb");
+    strcpy (strPathTemp, (char *) getenv ("HOME"));
+    strcat (strPathTemp, "/.fcitx/");
+    if (access (strPathTemp, 0))
+	mkdir (strPathTemp, S_IRWXU);
+    strcat (strPathTemp, TEMP_FILE);
+    fp = fopen (strPathTemp, "wb");
     if (!fp) {
-	fprintf (stderr, "޷ûʿ⣡\n");
+	fprintf (stderr, "޷ƴûʿ⣺%s\n", strPathTemp);
 	return;
     }
 
@@ -2266,25 +2320,33 @@ void SavePYUserPhrase (void)
     }
 
     fclose (fp);
+
+    strcpy (strPath, (char *) getenv ("HOME"));
+    strcat (strPath, "/.fcitx/");
+    strcat (strPath, PY_USERPHRASE_FILE);
+    if (access (strPath, 0))
+	unlink (strPath);
+    rename (strPathTemp, strPath);
 }
 
 void SavePYFreq (void)
 {
     int             i, j, k;
     char            strPath[PATH_MAX];
+    char            strPathTemp[PATH_MAX];
     FILE           *fp;
     PyFreq         *pPyFreq;
     HZ             *hz;
 
-    strcpy (strPath, (char *) getenv ("HOME"));
-    strcat (strPath, "/.fcitx/");
-    if (access (strPath, 0))
-	mkdir (strPath, S_IRWXU);
-    strcat (strPath, PY_FREQ_FILE);
+    strcpy (strPathTemp, (char *) getenv ("HOME"));
+    strcat (strPathTemp, "/.fcitx/");
+    if (access (strPathTemp, 0))
+	mkdir (strPathTemp, S_IRWXU);
+    strcat (strPathTemp, TEMP_FILE);
 
-    fp = fopen (strPath, "wb");
+    fp = fopen (strPathTemp, "wb");
     if (!fp) {
-	fprintf (stderr, "޷泣ôʱ\n");
+	fprintf (stderr, "޷泣ôʱ%s\n", strPathTemp);
 	return;
     }
     i = 0;
@@ -2321,6 +2383,13 @@ void SavePYFreq (void)
     }
 
     fclose (fp);
+
+    strcpy (strPath, (char *) getenv ("HOME"));
+    strcat (strPath, "/.fcitx/");
+    strcat (strPath, PY_FREQ_FILE);
+    if (access (strPath, 0))
+	unlink (strPath);
+    rename (strPathTemp, strPath);
 }
 
 /*
@@ -2330,16 +2399,17 @@ void SavePYIndex (void)
 {
     int             i, j, k, l, m;
     char            strPath[PATH_MAX];
+    char            strPathTemp[PATH_MAX];
     FILE           *fp;
 
-    strcpy (strPath, (char *) getenv ("HOME"));
-    strcat (strPath, "/.fcitx/");
-    if (access (strPath, 0))
-	mkdir (strPath, S_IRWXU);
-    strcat (strPath, PY_INDEX_FILE);
-    fp = fopen (strPath, "wb");
+    strcpy (strPathTemp, (char *) getenv ("HOME"));
+    strcat (strPathTemp, "/.fcitx/");
+    if (access (strPathTemp, 0))
+	mkdir (strPathTemp, S_IRWXU);
+    strcat (strPathTemp, PY_INDEX_FILE);
+    fp = fopen (strPathTemp, "wb");
     if (!fp) {
-	fprintf (stderr, "޷ļ\n");
+	fprintf (stderr, "޷ļ%s\n", strPathTemp);
 	return;
     }
 
@@ -2380,6 +2450,13 @@ void SavePYIndex (void)
     }
 
     fclose (fp);
+
+    strcpy (strPath, (char *) getenv ("HOME"));
+    strcat (strPath, "/.fcitx/");
+    strcat (strPath, PY_INDEX_FILE);
+    if (access (strPath, 0))
+	unlink (strPath);
+    rename (strPathTemp, strPath);
 }
 
 /*
@@ -2524,7 +2601,6 @@ INPUT_RETURN_VALUE PYGetLegendCandWords (SEARCH_MODE mode)
     char            strTemp[2];
     PyPhrase       *phrase;
 
-
     if (!strPYLegendSource[0])
 	return IRV_TO_PROCESS;
 
@@ -2657,9 +2733,6 @@ INPUT_RETURN_VALUE PYGetLegendCandWords (SEARCH_MODE mode)
 	if (i == 9)
 	    strTemp[0] = '0';
 	strcpy (messageDown[uMessageDown].strMsg, strTemp);
-#ifdef _USE_XFT
-	strcat (messageDown[uMessageDown].strMsg, " ");
-#endif
 	messageDown[uMessageDown++].type = MSG_INDEX;
 
 	strcpy (messageDown[uMessageDown].strMsg, PYLegendCandWords[i].phrase->strPhrase + PYLegendCandWords[i].iLength);
@@ -2678,7 +2751,7 @@ INPUT_RETURN_VALUE PYGetLegendCandWords (SEARCH_MODE mode)
     return IRV_DISPLAY_CANDWORDS;
 }
 
-Bool PYAddLengendCandWord (PyPhrase * phrase, INPUT_RETURN_VALUE mode)
+Bool PYAddLengendCandWord (PyPhrase * phrase, SEARCH_MODE mode)
 {
     int             i = 0, j;
 
@@ -2707,14 +2780,25 @@ Bool PYAddLengendCandWord (PyPhrase * phrase, INPUT_RETURN_VALUE mode)
 	    return True;
     }
 
-    if (iLegendCandWordCount == iMaxCandWord) {
-	for (j = 0; j < i; j++) {
-	    PYLegendCandWords[j].phrase = PYLegendCandWords[j + 1].phrase;
-	    PYLegendCandWords[j].iLength = PYLegendCandWords[j + 1].iLength;
+    if (mode == SM_PREV) {
+	if (iLegendCandWordCount == iMaxCandWord) {
+	    for (j = 0; j < i; j++) {
+		PYLegendCandWords[j].phrase = PYLegendCandWords[j + 1].phrase;
+		PYLegendCandWords[j].iLength = PYLegendCandWords[j + 1].iLength;
+	    }
+	}
+	else {
+	    for (j = iLegendCandWordCount; j > i; j--) {
+		PYLegendCandWords[j].phrase = PYLegendCandWords[j - 1].phrase;
+		PYLegendCandWords[j].iLength = PYLegendCandWords[j - 1].iLength;
+	    }
 	}
     }
     else {
-	for (j = iLegendCandWordCount; j > i; j--) {
+	j = iLegendCandWordCount;
+	if (iLegendCandWordCount == iMaxCandWord)
+	    j--;
+	for (; j > i; j--) {
 	    PYLegendCandWords[j].phrase = PYLegendCandWords[j - 1].phrase;
 	    PYLegendCandWords[j].iLength = PYLegendCandWords[j - 1].iLength;
 	}
